/****************************************************************************/
/*!
  \file     encoder.c

  \brief    Funktionen zur Vorgabe von Fahrstrecken und Drehungen.

  \see      Defines zum setzen von Port's und Konfigurationen in asuro.h\n
            TRUE, FALSE, LEFT, RIGHT

  \version  V--- -
            KEINE original Version von der ASURO CD vom DLR\n
  \version  V--- - bis zum 07.01.2007 - \n
            Bitte in Datei CHANGELOG nachsehen.\n
            Das Grundgeruest dieser Funktionen wurde von stochri erstellt.\n
  \version  V001 - 13.01.2007 - m.a.r.v.i.n\n
            +++ Alle Funktionen\n
            Zerlegte Sourcen in einzelne Dateien fuer eine echte Library.
  \version  V002 - 27.01.2007 - Sternthaler\n
            +++ Alle Funktionen\n
            Kommentierte Version (KEINE Funktionsaenderung)
  \version  V003 - 20.02.2007 - m.a.r.v.i.n\n
            Korrekturfakturen aus myasuro.h verwenden
*****************************************************************************/
/*****************************************************************************
*                                                                            *
*   This program is free software; you can redistribute it and/or modify     *
*   it under the terms of the GNU General Public License as published by     *
*   the Free Software Foundation; either version 2 of the License, or        *
*   any later version.                                                       *
*                                                                            *
*****************************************************************************/
#include "asuro.h"
#include "myasuro.h"


/***************************************************************************
* void Go(int distance, int speed)
*
* Go distance in mm. Attention: EncoderInit() has to be called first.
*
* the driven distance depends a little bit from the floor friction
* limitations: maximum distance +-32m
* possible problems: in full sunlight the encoder sensors may be disturbed
*
*   input
* distance: postiv->go forward ; negativ-> go backward
*   speed: sets motorspeed
*
* last modification:
*   Ver.     Date         Author           Comments
*   -------  ----------   --------------   ---------------------------------
*   sto1     29.07.2005   stochri          motorfunction
*   And1     31.07.2005   Andun            added speed and Odometrie
*   sto2     31.10.2006   stochri          distance in mm
*   -------  ----------   --------------   ---------------------------------
*
***************************************************************************/
/****************************************************************************/
/*!
  \brief
  Faehrt eine bestimmte Strecke mit einer bestimmten Geschwindigkeit.
  (Autor: stochri)\n
  Benutzt die Odometrie Sensoren im Interrupt Betrieb.\n
  Vor dem ersten Aufruf muss deshalb EncoderInit() aufgerufen werden.

  \param[in]
  distance Distanz in mm (- rueckwaerts, + = vorwaerts)
  \param[in]
  speed Geschwindigkeit (Wertebereich 0...255)

  \return
  nichts

  \see
  In der globale Variable encoder, werden die Hell-/Dunkelwechsel der Encoder-\n
  scheiben im Interruptbetrieb gezaehlt.

  \par  Funktionsweise:
  Anhand des Vorzeichens der zu fahrenden Strecke wird ueber MotorDir() die\n
  Richtung gesetzt.\n
  Die zu fahrende Strecke wird in Hell-/Dunkelwechsel (Ticks) umgerechnet.\n
  Die Geschwindigkeit wird mit MotorSpeed() eingestellt.\n
  In einer Programmschleife werden nun die Ticks der linken Seiten gezaehlt.\n
  Wird die berechnete Vorgabe erreicht, werden die Motoren gestoppt, und die\n
  Funktion wird beendet.\n
  Damit die Fahrt auch gerade ausgefuehrt wird, werden in der Schleife immer\n
  die Ticks der rechten und linken Seite verglichen und die Geschwindigkeit\n
  einer Seite so angepasst, dass der Asuro geradeaus faehrt. Dabei wird\n
  darauf geachtet, dass die vorgegebene Geschwindigkeit moeglichst\n
  eingehalten wird.

  \par  Hinweis:
  Die Berechnung der zu fahrenden Ticks beruht auf der Annahme, dass die\n
  Anzahl der schwarzen Teilstuecke und die Raddurchmesser wie bei stochri sind.\n
  (Sternthaler) Vermutung, dass der Hersteller unterschiedlich grosse Raeder\n
  ausgeliefert hat, da die Berechnung in dieser Form bei Sternthaler nicht\n
  funktioniert.

  \par  Beispiel:
  (Nur zur Demonstration der Parameter/Returnwerte)
  \code
  // Laesst den Asuro ein Quadrat von 200 mm fahren,
  // bei einer Geschwindigkeit von 150.
  EncoderInit ();
  for (int i = 0; i < 4; i++)
  {
    Go (200, 150);
    Turn (90, 150);
  }
  \endcode
*****************************************************************************/
void Go (
  int distance,
  int speed)
{
  uint32_t enc_count;
  int tot_count = 0;
  int diff = 0;
  int l_speed = speed, r_speed = speed;

  // calculation tics/mm
  enc_count=abs(distance)*10000L;
  enc_count/=MY_GO_ENC_COUNT_VALUE;
  EncoderSet(0,0); // reset encoder

  MotorSpeed(l_speed,r_speed);
  if (distance<0) MotorDir(RWD,RWD);
  else MotorDir(FWD,FWD);

  while (tot_count<enc_count)
  {
    tot_count += encoder[LEFT];
    diff = encoder[LEFT] - encoder[RIGHT];
    if (diff > 0)
    { //Left faster than right
      if ((l_speed > speed) || (r_speed > 244)) l_speed -= 10;
      else r_speed += 10;
    }
    if (diff < 0)
    { //Right faster than left
      if ((r_speed > speed) || (l_speed > 244)) r_speed -= 10;
      else l_speed += 10;
    }
    EncoderSet(0,0); // reset encoder
    MotorSpeed(l_speed,r_speed);
    Msleep(1);
  }
  MotorDir(BREAK,BREAK);
  Msleep(200);
}



/***************************************************************************
* void Turn(int degree, int speed)
*
*   Turn given angle. Attention: EncoderInit() has to be called first.
*
*   input
* degree: postiv->turn right ; negativ-> turn left
*
* last modification:
*   Ver.     Date         Author           Comments
*   -------  ----------   --------------   ---------------------------------
*   sto1     29.07.2005   stochri          motorfunction
*   And1     07.08.2005   Andun            Added Odometrie function
*   sto2     31.10.2006   stochri          added comments, corrected enc_count initialisation
*   -------  ----------   --------------   ---------------------------------
*
***************************************************************************/
/****************************************************************************/
/*!
  \brief
  Dreht um einen bestimmten Winkel mit einer bestimmten Geschwindigkeit.
  (Autor: stochri)\n
  Benutzt die Odometrie Sensoren im Interrupt Betrieb.\n
  Vor dem ersten Aufruf muss deshalb EncoderInit() aufgerufen werden.

  \param[in]
  degree Winkel (- rechts, + links)
  \param[in]
  speed Geschwindigkeit (Wertebereich 0...255)

  \return
  nichts

  \see
  Funktion Go(), mit dem Unterschied, dass hier anhand des Vorzeichens der\n
  uebergebenen Geschwindigkeit mal eine rechts- bzw. links-Drehung erzeugt wird.

  \see
  In der globale Variable encoder, werden die Hell-/Dunkelwechsel der\n
  Encoderscheiben im Interruptbetrieb gezaehlt.

  \par  Hinweis:
  Die Berechnung der zu fahrenden Ticks beruht auf der Annahme, dass die\n
  Anzahl der schwarzen Teilstuecke und die Raddurchmesser wie bei stochri sind.\n
  (Sternthaler) Vermutung, dass der Hersteller unterschiedlich grosse Raeder\n
  ausgeliefert hat, da die Berechnung in dieser Form bei Sternthaler nicht\n
  funktioniert.

  \par Beispiel:
  (Nur zur Demonstration der Parameter/Returnwerte)
  \code
  // Laesst den Asuro ein Quadrat von 200 mm fahren,
  // bei einer Geschwindigkeit von 150.
  EncoderInit ();
  for (int i = 0; i < 4; i++)
  {
    Go (200, 150);
    Turn (90, 150);
  }
  \endcode
*****************************************************************************/
void Turn (
  int degree,
  int speed)
{
  long enc_count;
  enc_count=abs(degree)*MY_TURN_ENC_COUNT_VALUE;
  enc_count /= 360L;

  int tot_count = 0;
  int diff = 0;
  int l_speed = speed, r_speed = speed;


  EncoderSet(0,0);    // reset encoder

  MotorSpeed(l_speed,r_speed);
  if (degree<0) MotorDir(RWD,FWD);
  else MotorDir(FWD,RWD);

  while (tot_count<enc_count)
  {
    tot_count += encoder[LEFT];
    diff = encoder[LEFT] - encoder[RIGHT];
    if (diff > 0)
    { //Left faster than right
      if ((l_speed > speed) || (r_speed > 244)) l_speed -= 10;
      else r_speed += 10;
    }
    if (diff < 0)
    { //Right faster than left
      if ((r_speed > speed) || (l_speed > 244)) r_speed -= 10;
      else l_speed += 10;
    }
    EncoderSet(0,0);    // reset encoder
    MotorSpeed(l_speed,r_speed);
    Msleep(1);
  }
  MotorDir(BREAK,BREAK);
  Msleep(200);
}

