/****************************************************************************/
/*!
  \file print.c

  \brief
  Print Ausgabe Funktionen

  Die Ausgabe der Zeichen erfolgt ueber die IR LED an der seriellen Schnittstelle.\n
  Ueber den IR Transceiver koennen die Zeichen empfangen und in einem Terminalprogramm\n
  vom PC dargestellt werden.\n
  Dazu muss bei den Einstellungen des Terminalprogramms die COM Schnittstelle\n 
  ausgewaehlt werden, an der der IR Transceiver haengt.\n 
  Die weiteren Schnittstellenparameter sind:\n 
  2400 Baud, 8 Databits, no parity, 1 Stopbit, Hardware Handshake none\n
  
  \par Hinweis:
  Wer den USB IR Transceiver mit einem Terminalprogramm benutzen will,\n
  muss dazu die aktuellen USB Treiber von der FTDI Homepage verwenden.\n
  Diese CDM Treiber enthalten sowohl die VCP Treiber,\n
  die das Terminalprogramm verwendet, als auch die D2XX Treiber,\n
  die das Flasher Tool verwendet.\n
  Zu finden sind die Treiber unter http://www.ftdichip.com/FTDrivers.htm.  

  \version  V--- - 10.11.2003 - Jan Grewe - DLR\n
            Original Version von der ASURO CD\n
  \version  V--- - bis zum 07.01.2007 - \n
            Bitte in Datei CHANGELOG nachsehen.\n
  \version  V001 - 13.01.2007 - m.a.r.v.i.n\n
            +++ Alle Funktionen\n
            Zerlegte Sourcen in einzelne Dateien fuer eine echte Library.
  \version  V002 - 09.02.2007 - m.a.r.v.i.n\n
            +++ PrintLong ()  NEU
            Ausgabe eines Long Wertes als String ueber die serielle
            Schnittstelle.\n
            +++ Alle Funktionen\n
            Kommentierte Version\n
  \version  V003 - 14.02.2007 - Sternthaler\n
            +++ Alle Funktionen\n
            Kommentare einheitlicher zu den anderen Sourcen.\n
*****************************************************************************/
/*****************************************************************************
*                                                                            *
*   This program is free software; you can redistribute it and/or modify     *
*   it under the terms of the GNU General Public License as published by     *
*   the Free Software Foundation; either version 2 of the License, or        *
*   any later version.                                                       *
*                                                                            *
*****************************************************************************/
#include "asuro.h"



/****************************************************************************/
/*!
  \brief
  Ausgabe eines Integer Wertes als String ueber die serielle Schnittstelle.

  \param[in]
  wert Auszugebender Integer Wert (16Bit)

  \author   Robotrixer, marvin

  \version  beta - 31.03.2005 - Robotrixer\n
            first implementation
  \version  2.60 - 28.09.2005 - m.a.r.v.i.n\n
            strlen verwenden, anstelle fester Laenge
  \version  2.61 - 20.11.2006 - m.a.r.v.i.n\n
            Initialisierung text String kann zu Fehler\n
            beim Flashen mit RS232/IR Adapter fuehren\n
            (Bug report von francesco)
  \version  2.70b- 07.01.2007 - m.a.r.v.i.n\n
            SerPrint Funktion anstelle SerWrite verwenden
  \version  2.70rc2- 09.02.2007 - m.a.r.v.i.n\n
            Text Laenge auf 7 erhoeht, fuer Ausgabe negativer Werte
            (Bug Report von HermannSW)

  \par  Beispiel:
  (Nur zur Demonstration der Parameter/Returnwerte)
  \code
  // Beispiel fuer zeilenweise Ausgabe der Encoder Werte
  EncoderInit();
  While(1)
  {
    PrintInt (encoder [0]);
    SerWrite ("   ", 3);
    PrintInt (encoder [1]);
    SerPrint ("\n\r");
    MSleep (500); //0,5sek. warten
  }
  \endcode
*****************************************************************************/
void PrintInt (
  int wert)
{
  char text [7];                        // "-12345"

  itoa (wert, text, 10);
  SerPrint (text);
}



/****************************************************************************/
/*!
  \brief
  Ausgabe eines Long Wertes als String ueber die serielle Schnittstelle.

  \param[in]
  wert Auszugebender Long Wert (32Bit)

  \author HermannSW, marvin

  \version  2.70rc2 - 09.02.2007    m.a.r.v.i.n\n
            first implementation
            (Idee von HermannSW)

  \par  Beispiel:
  (Nur zur Demonstration der Parameter/Returnwerte)
  \code
  // Beispiel fuer Ausgabe der Batterie Werte
  long data;
  While(1)
  {
    data = Batterie ();
    PrintLong (data);
    SerPrint ("\n\r");
    MSleep (500); //0,5sek. warten
  }
  \endcode
*****************************************************************************/
void PrintLong (
  long wert)
{
  char text [12];                       // '-'1234567891'\0'

  ltoa (wert, text, 10);
  SerPrint (text);
}



/****************************************************************************/
/*!
  \brief
  Sendet einen einzelnen Character ber die serielle Schnittstelle

  \param[in]
  zeichen auszugebendes Zeichen

  \author   stochri

  \version  sto1 - 07.01.2006   stochri\n
            first implementation

  \par  Beispiel:
  (Nur zur Demonstration der Parameter/Returnwerte)
  \code
  // 'wunderbaren' Text als einzelne Zeichen senden.
  UartPutc ('H');
  UartPutc ('e');
  UartPutc ('l');
  UartPutc ('l');
  UartPutc ('o');
  \endcode
*****************************************************************************/
void UartPutc (
  unsigned char zeichen)
{
  UCSRB  = 0x08;                        // enable transmitter
  UCSRA |= 0x40;                        // clear transmitter flag
  while (!(UCSRA & 0x20))               // wait for empty transmit buffer
    ;
  UDR = zeichen;
  while (!(UCSRA & 0x40))               // Wait for transmit complete flag (TXC)
    ;
}



/****************************************************************************/
/*!
  \brief
  Sendet einen null-terminierten String ueber die serielle Schnittstelle.

  \param[in]
  data null-terminierter String

  \author   stochri

  \version  sto1 - 07.01.2007 - stochri\n
            first implementation

  \par  Beispiel:
  (Nur zur Demonstration der Parameter/Returnwerte)
  \code
  // Beispiel fuer SerPrint
  SerPrint ("Hallo ASURO!\r\n");
  \endcode
*****************************************************************************/
void SerPrint (
  unsigned char *data)
{
  unsigned char i = 0;

  while (data [i] != 0x00)
    UartPutc (data [i++]);
}
